using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Collections.ObjectModel;
using System.Windows.Input;
using SharpCharts.Base.Common;
using System.Xml.Linq;

namespace SharpCharts.Base.Data
{
    public class CustomizableImportType : DefaultImportBase
    {
        DateTime date;
        TimeSpan time;
        string description;
    	public CustomizableImportType() : base()
    	{
    		this.DataItems = new ObservableCollection<DataType>();
    		this.AddItemCommand = new RelayCommand(OnAddItem);
    		this.RemoveItemCommand = new RelayCommand<DataType>(OnRemoveItem);
    		this.SaveSettingsCommand = new RelayCommand(OnSaveSettings);
    		this.LoadSettingsCommand = new RelayCommand(OnLoadSettings);
    	}
    	
		public override string Description 
		{
			get 
			{
				return this.description; 
			}
		}
    
        public override BuiltDataType BuiltDataType
        {
            get { return this.BuiltType; }
        }

       private int dataIndex;

        protected override bool? GetNextDataPoint()
        {
            if (base.Instrument == null)
                return false;

            if (base.data == null)
                return false;

            if (this.dataIndex >= base.data.Length)
                return false;
                
            string[] quotes = base.data[this.dataIndex++].Split(new string[] { this.Separator }, StringSplitOptions.RemoveEmptyEntries);

			if (this.dataIndex <= this.LinesToSkip)
            	return null;	//will continue the loop
            	
			this.date = DateTime.MinValue;
			this.time = TimeSpan.Zero;

            if (quotes.Length != this.DataItems.Count)
            	return false;
            
            for (int i = 0; i < this.DataItems.Count; i++)
            {
              	if (!this.TryGetValue(this.DataItems[i], quotes[i]))
               	{
               		return false;                		
               	}
            }

			base.Time = this.date.Add(this.time);
            
            return true;
        }
        
        private bool TryGetValue(DataType dataType, string value)
        {
        	double @double;
        	long @long;
        	
        	switch (dataType)
        	{
        		case DataType.Date:
        			if (DateTime.TryParseExact(value, new string[] { this.DateFormat }, System.Globalization.CultureInfo.InvariantCulture, System.Globalization.DateTimeStyles.None, out this.date))
        			{
        				return true;
        			}
        			break;
        		case DataType.Time:
        			if (TimeSpan.TryParseExact(value, new string[] { this.TimeFormat }, System.Globalization.CultureInfo.InvariantCulture, out this.time))
        			{
        				return true;
        			}
        			break;
        		case DataType.Open:
        			if (double.TryParse(value, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture, out @double))
        			{
        				this.Open = @double;
        				return true;
        			}
        			break;
        		case DataType.High:
        			if (double.TryParse(value, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture, out @double))
        			{
        				this.High = @double;
        				return true;
        			}
        			break;
        		case DataType.Low:
        			if (double.TryParse(value, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture, out @double))
        			{
        				this.Low = @double;
        				return true;
        			}
        			break;
        		case DataType.Close:
        			if (double.TryParse(value, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture, out @double))
        			{
        				this.Close = @double;
        				return true;
        			}
        			break;
        		case DataType.Volume:
        			if (long.TryParse(value, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture, out @long))
        			{
        				this.Volume = @long;
        				return true;
        			}
        			break;
        		case DataType.OpenInterest:
        			if (long.TryParse(value, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture, out @long))
        			{
        				this.OI = @long;
        				return true;
        			}
        			break;
        		case DataType.Bid:
        			if (double.TryParse(value, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture, out @double))
        			{
        				this.Bid = @double;
        				return true;
        			}
        			break;
        		case DataType.Ask:
        			if (double.TryParse(value, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture, out @double))
        			{
        				this.Ask = @double;
        				return true;
        			}
        			break;
        		case DataType.Skip:
        			return true;
        	}
        	
        	return false;
        }

        

        protected override void OnStateChanged()
        {
            switch (base.State)
            {
                case SharpCharts.Base.SharpScript.State.Finalize:
                    break;
                case SharpCharts.Base.SharpScript.State.Initialize:
                	
                	this.Filter = string.Format("Data Files (*.{0})|*.{0}", this.FileExtension);
                	this.DefaultExtension = string.Format("*.{0}", this.FileExtension);
                    this.InitialDirectory = Common.SharpFolder.UserDirectoryPath;
                
                    break;
            }
        }
        
        
        private void OnSaveSettings()
    	{
    		var xmlFile = new XElement("Settings",
    			new XElement("FileExtension", this.FileExtension),
    			new XElement("BuiltType", this.BuiltType),
    			new XElement("Separator", this.Separator),
    			new XElement("DateFormat", this.DateFormat),
    			new XElement("TimeFormat", this.TimeFormat),
    			new XElement("LinesToSkip", this.LinesToSkip),
    			new XElement("DataItems", 
    				from item in this.DataItems
    					select new XElement("DataType", item))
    			);
    			
    		xmlFile.Save(SharpFolder.Misc + this.GetType().Name + ".xml");
    		
    	}
    	
    	private void OnLoadSettings()
    	{
    		string filename = SharpFolder.Misc + this.GetType().Name + ".xml";
    		if (!System.IO.File.Exists(filename))
    			return;
    			
    		this.DataItems.Clear();
    		
    		XDocument document = XDocument.Load(filename);
    		if (document != null)
    		{
    			XElement element = document.Root.Element("FileExtension");
    			if (element != null)
    			{
    				this.FileExtension = element.Value;
    			}
    			
    			element = document.Root.Element("BuiltType");
    			if (element != null)
    			{
    				BuiltDataType builtType;
    				if (Enum.TryParse(element.Value, out builtType))
    				{
    					this.BuiltType = builtType;
    				}
    			}
    			
    			element = document.Root.Element("DateFormat");
    			if (element != null)
    			{
    				this.DateFormat = element.Value;
    			}
    			
    			element = document.Root.Element("TimeFormat");
    			if (element != null)
    			{
    				this.TimeFormat = element.Value;
    			}
    			
    			element = document.Root.Element("LinesToSkip");
    			if (element != null)
    			{
    				int @int;
    				if (int.TryParse(element.Value, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture, out @int))
    				{
    					this.LinesToSkip = @int;
    				}
    			}
    			
    			element = document.Root.Element("DataItems");
    			if (element != null)
    			{
    				IEnumerable<XElement> dataItems = element.Elements("DataType");
    				if (dataItems != null)
    				{
	    				foreach	(var item in dataItems)
    					{
    						DataType dataType;
    						if (Enum.TryParse(item.Value, out dataType))
	    						this.DataItems.Add(dataType);						
    					}
    				}
    			}
    		}
    		
    		PopulateDescription();
    	}
    	
    	private void OnAddItem()
    	{
    		if (Enum.IsDefined(typeof(DataType), this.SelectedDataType))
    		{
    			this.DataItems.Add(this.SelectedDataType);
    			PopulateDescription();
    		}
    	}
    	
    	private void OnRemoveItem(DataType dataType)
    	{
    		this.DataItems.Remove(dataType);
    		PopulateDescription();
    	}
    	
    	private void PopulateDescription()
    	{
    		StringBuilder sb = new StringBuilder();
    		sb.AppendLine(string.Format("Parses {0} data.", this.BuiltDataType));
			    		
    		for	(int i = 0; i < this.DataItems.Count; i++)
    		{
    			if (i == this.DataItems.Count - 1)
	    			sb.Append("<" + this.DataItems[i].ToString() + ">");
	    		else sb.Append("<" + this.DataItems[i] + ">" + this.Separator);
    		}
    		
    		this.description = sb.ToString();
    		
    		NotifyPropertyChanged("Description");
    	}
        
        private BuiltDataType builtType = BuiltDataType.Minute;
        public BuiltDataType BuiltType
        {
        	get { return builtType; }
        	set
        	{
        		builtType = value;
        		NotifyPropertyChanged("BuiltType");
        		PopulateDescription();
        	}
        }
        
        private string fileExtension = "txt";
        
        public string FileExtension
        {
        	get { return fileExtension; }
        	set
        	{
        		fileExtension = value;
        		NotifyPropertyChanged("FileExtension");
        	}
        }
        
        private string separator = ",";
        public string Separator 
        {
        	get { return separator; }
        	set 
        	{
        		separator = value;
        		NotifyPropertyChanged("Separator");
        		PopulateDescription();
        	}
        }
        
        private string dateFormat = "yyyyMMdd";
        
        public string DateFormat
        {
        	get { return dateFormat; }
        	set 
        	{
        		dateFormat = value;
        		NotifyPropertyChanged("DateFormat");
        	}
        }
        
        private string timeFormat = "hh:mm:ss";
        
        public string TimeFormat 
        {
        	get { return timeFormat; }
        	set 
        	{
        		timeFormat = value;
        		NotifyPropertyChanged("TimeFormat");
        	}
        }
        
        public DataType[] DataTypes
        {
        	get 
        	{
        		return new DataType[] 
        		{ 
        			DataType.Date, 
        			DataType.Time, 
        			DataType.Open, 
        			DataType.High, 
        			DataType.Low, 
        			DataType.Close, 
        			DataType.Volume, 
        			DataType.OpenInterest,
        			DataType.Bid,
        			DataType.Ask,
        			DataType.Skip
        		};
        	}
        }
        
        public ObservableCollection<DataType> DataItems { get; private set; }
        
        private DataType selectedDataType = DataType.Close;
        
        public DataType SelectedDataType 
        {
        	get { return selectedDataType; }
        	set
        	{
        		selectedDataType = value;
        		NotifyPropertyChanged("SelectedDataType");
        	}
        }
        
        private int linesToSkip = 0;
        
        public int LinesToSkip
        {
        	get { return linesToSkip; }
        	set
        	{
        		linesToSkip = value;
        		NotifyPropertyChanged("LinesToSkip");
        	}
        }
        
        
        public ICommand AddItemCommand { get; private set; }
        public ICommand RemoveItemCommand { get; private set; }
        public ICommand SaveSettingsCommand { get; private set; }
        public ICommand LoadSettingsCommand { get; private set; }
        
    }
    
    public enum DataType
    {
    	Date, Time, Open, High, Low, Close, Volume, OpenInterest, Bid, Ask, Skip
    }
    
}